import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def estimator_command_factory(args):
    return EstimatorCommand(args.num_train_samples, args.project_name)


class EstimatorCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        estimator_parser = parser.add_parser(
            "estimate", description="💰 Fetches estimated cost for a project in AutoNLP"
        )
        estimator_parser.add_argument(
            "--num_train_samples", type=int, required=True, help="Number of training samples"
        )
        estimator_parser.add_argument(
            "--project_name",
            type=str,
            required=True,
            help="The project's name",
        )
        estimator_parser.set_defaults(func=estimator_command_factory)

    def __init__(self, num_train_samples: int, proj_name: str):
        self._num_train_samples = num_train_samples
        self._proj_name = proj_name

    def run(self):
        from ..autonlp import AutoNLP

        client = AutoNLP()
        try:
            resp = client.estimate(num_train_samples=self._num_train_samples, proj_name=self._proj_name)
            print(f"Cost range: {resp['cost_min']} - {resp['cost_max']} USD")
        except ValueError:
            logger.error("Something bad happened. Couldn't make the estimate.")
            sys.exit(1)



import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from typing import Optional

from loguru import logger

from autonlp.validation import InvalidColMappingError, InvalidFileError

from ..utils import BOLD_TAG as BLD
from ..utils import RED_TAG as RED
from ..utils import RESET_TAG as RST
from . import BaseAutoNLPCommand
from .common import COL_MAPPING_ARG_HELP, COL_MAPPING_HELP


def upload_command_factory(args):
    return UploadCommand(
        name=args.project,
        split=args.split,
        col_mapping=args.col_mapping,
        files=args.files,
        path_to_audio=args.path_to_audio,
    )


class UploadCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        upload_parser = parser.add_parser(
            "upload",
            description="📡 Upload files to AutoNLP! The files will be uploaded to a private dataset on your 🤗.co account",
            epilog=COL_MAPPING_HELP,
            formatter_class=RawTextHelpFormatter,
        )
        upload_parser.add_argument(
            "--project", type=str, default=None, required=True, help="The name of the project to upload files to"
        )
        upload_parser.add_argument(
            "--files", type=str, required=True, help="Paths to the files to upload, comma-separated"
        )
        upload_parser.add_argument(
            "--split",
            type=str,
            default=None,
            required=True,
            metavar="SPLIT",
            help=f"The files' split, must be one of {['train', 'valid']}",
            choices=["train", "valid"],
        )
        upload_parser.add_argument(
            "--col_mapping",
            type=str,
            default=None,
            required=True,
            help=COL_MAPPING_ARG_HELP,
        )
        upload_parser.add_argument(
            "--path_to_audio",
            type=str,
            default=None,
            required=False,
            help=(
                "Required for speech recognition task. "
                f"Comma-separated paths to {BLD}directories{RST} containing audio files"
            ),
        )
        upload_parser.set_defaults(func=upload_command_factory)

    def __init__(self, name: str, split: str, col_mapping: str, files: str, path_to_audio: Optional[str]):
        self._name = name
        self._split = split
        self._col_mapping = col_mapping
        self._files = files
        self._path_to_audio = path_to_audio

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Uploading files for project: {self._name}")
        client = AutoNLP()
        try:
            project = client.get_project(name=self._name)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        splits = self._col_mapping.split(",")
        col_maps = {}
        for s in splits:
            k, v = s.split(":")
            col_maps[k] = v
        logger.info(f"Mapping: {col_maps}")

        files = self._files.split(",")
        try:
            project.upload(filepaths=files, split=self._split, col_mapping=col_maps, path_to_audio=self._path_to_audio)
            print(
                "🎉 Yupee! Your files have been uploaded.\n"
                f"Once you're done, starting a training here: {RED}autonlp train --project {project.name}{RST}"
            )
        except ValueError as err:
            logger.error("❌ Something went wrong!")
            logger.error("Details:")
            logger.error(str(err))
        except FileNotFoundError as err:
            logger.error("❌ One path you provided is invalid!")
            logger.error("Details:")
            logger.error(str(err))
        except InvalidFileError as err:
            logger.error("❌ Sorry, AutoNLP is not able to process the files you want to upload")
            logger.error("Details:")
            for line in str(err).splitlines():
                logger.error(line)
        except InvalidColMappingError as err:
            logger.error("❌ The column mapping you provided is incorrect!")
            logger.error("Details:")
            for line in str(err).splitlines():
                logger.error(line)



from argparse import ArgumentParser

from loguru import logger

from ..languages import SUPPORTED_LANGUAGES
from ..tasks import TASKS
from ..utils import RED_TAG as RED
from ..utils import RESET_TAG as RST
from . import BaseAutoNLPCommand


def create_project_command_factory(args):
    if args.max_models <= 0:
        raise ValueError("max_models cannot be 0 or negative")
    if args.max_models > 150:
        raise ValueError("Please choose a value from 1 to 150 for max_models")
    if args.hub_model is None and args.language == "unk":
        raise ValueError("Please provide the `language` parameter")
    return CreateProjectCommand(args.name, args.task, args.language, args.max_models, args.hub_model)


class CreateProjectCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        create_project_parser = parser.add_parser("create_project", description="✨ Creates a project in AutoNLP.")
        create_project_parser.add_argument("--name", type=str, default=None, required=True, help="The project's name")
        create_project_parser.add_argument(
            "--task",
            metavar="TASK",
            type=str,
            default=None,
            required=True,
            help=f"The project's task type, one of: {list(TASKS.keys())}",
            choices=list(TASKS.keys()),
        )
        create_project_parser.add_argument(
            "--language",
            type=str,
            default="unk",
            required=False,
            metavar="LANGUAGE",
            help="The project's language. Please check supported languages in AutoNLP documentation.",
            choices=SUPPORTED_LANGUAGES,
        )
        create_project_parser.add_argument(
            "--max_models",
            type=int,
            default=10,
            required=True,
            metavar="MAX_MODELS",
            help="Maximum number of models you want to train in this project. More models => higher chances of getting awesome models. Also, more models => higher expenses",
        )
        create_project_parser.add_argument(
            "--hub_model",
            type=str,
            default=None,
            required=False,
            metavar="HUB_MODEL",
            help="Provide model from hub that you want to finetune. E.g. abhishek/my_awesome_model. Note that if you provide a hub model, AutoNLP will ignore `language` parameter and disable model search.",
        )
        create_project_parser.set_defaults(func=create_project_command_factory)

    def __init__(self, name: str, task: str, language: str, max_models: int, hub_model: str = None):
        self._name = name
        self._task = task
        self._lang = language
        self._max_models = max_models
        self._hub_model = hub_model

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Creating project: {self._name} with task: {self._task}")
        client = AutoNLP()
        project = client.create_project(
            name=self._name,
            task=self._task,
            language=self._lang,
            max_models=self._max_models,
            hub_model=self._hub_model,
        )
        print(project)
        print(f'Upload files to your project: {RED}autonlp upload --project "{project.name}"{RST}')



from argparse import ArgumentParser

from ..utils import RED_TAG as RED
from ..utils import RESET_TAG as RST
from . import BaseAutoNLPCommand


def login_command_factory(args):
    return LoginCommand(api_key=args.api_key)


class LoginCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        login_parser = parser.add_parser("login", description="🔐 Logs you in AutoNLP!")
        login_parser.add_argument(
            "--api-key",
            type=str,
            required=True,
            help="Your API key. You can find it under you user profile settings on huggingface.co!",
        )
        login_parser.set_defaults(func=login_command_factory)

    def __init__(self, api_key: str):
        self._api_key = api_key

    def run(self):
        from ..autonlp import AutoNLP

        client = AutoNLP()
        client.login(token=self._api_key)

        print(f"Welcome to 🤗 AutoNLP! Start by creating a project: {RED}autonlp create_project{RST}")



import sys
from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def project_info_command_factory(args):
    return ProjectInfoCommand(args.name, args.is_eval)


class ProjectInfoCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        project_info_parser = parser.add_parser(
            "project_info", description="♻ Refreshes and displays information about a project"
        )
        project_info_parser.add_argument("--name", type=str, default=None, required=True, help="The project's name")
        project_info_parser.add_argument(
            "--is_eval", action="store_true", help="Use `--is_eval` flag if this is an evaluation project"
        )
        project_info_parser.set_defaults(func=project_info_command_factory)

    def __init__(self, name: str, is_eval: bool):
        self._name = name
        self._is_eval = is_eval

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Fetching info for project: {self._name}")
        client = AutoNLP()
        try:
            project = client.get_project(name=self._name, is_eval=self._is_eval)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        print(project)



import argparse
import sys

from loguru import logger
from requests import HTTPError

from .. import __version__
from .benchmark import CreateBenchmarkCommand
from .create_project import CreateProjectCommand
from .estimator import EstimatorCommand
from .evaluate import CreateEvaluationCommand
from .list_projects import ListProjectsCommand
from .login import LoginCommand
from .metrics import MetricsCommand
from .predict import PredictCommand
from .project_info import ProjectInfoCommand
from .train import TrainCommand
from .upload import UploadCommand


def main():
    parser = argparse.ArgumentParser(
        "AutoNLP CLI",
        usage="autonlp <command> [<args>]",
        epilog="For more information about a command, run: `autonlp <command> --help`",
    )
    parser.add_argument("--version", "-v", help="Display AutoNLP version", action="store_true")
    commands_parser = parser.add_subparsers(help="commands")

    # Register commands
    LoginCommand.register_subcommand(commands_parser)
    CreateProjectCommand.register_subcommand(commands_parser)
    ProjectInfoCommand.register_subcommand(commands_parser)
    UploadCommand.register_subcommand(commands_parser)
    TrainCommand.register_subcommand(commands_parser)
    MetricsCommand.register_subcommand(commands_parser)
    ListProjectsCommand.register_subcommand(commands_parser)
    PredictCommand.register_subcommand(commands_parser)
    EstimatorCommand.register_subcommand(commands_parser)
    CreateEvaluationCommand.register_subcommand(commands_parser)
    CreateBenchmarkCommand.register_subcommand(commands_parser)

    args = parser.parse_args()

    if args.version:
        print(__version__)
        exit(0)

    if not hasattr(args, "func"):
        parser.print_help()
        exit(1)

    command = args.func(args)

    try:
        command.run()
    except HTTPError as err:
        status_code = err.response.status_code
        details = err.response.json().get("detail")
        if status_code == 403:
            logger.error("🛑 Forbidden operation")
            if details:
                logger.error(details)
        else:
            logger.error("❌ Oops! Something failed in AutoNLP backend..")
            if not details:
                details = err.response.text
            logger.error(f"Error code: {status_code}; Details: '{details}'")
        sys.exit(1)


if __name__ == "__main__":
    main()



from dataclasses import dataclass
from datetime import datetime
from typing import Tuple

from datasets.load import import_main_class, prepare_module

from .utils import BOLD_TAG, GREEN_TAG, RESET_TAG, YELLOW_TAG, get_task


EVAL_JOB_STATUS = (
    ("⌚", "queued"),
    ("🚀", "start"),
    ("⚙", "data_munging"),
    ("🏃", "model_evaluating"),
    ("✅", "success"),
    ("❌", "failed"),
)


@dataclass
class Evaluate:
    _token: str
    user: str
    task: str
    evaluation_id: int
    created_at: datetime
    updated_at: datetime
    dataset: str
    model: str

    @classmethod
    def from_json_resp(cls, json_resp: dict, token: str):
        task = get_task(json_resp["task"])
        return cls(
            _token=token,
            user=json_resp["username"],
            task=task,
            evaluation_id=json_resp["id"],
            created_at=datetime.fromisoformat(json_resp["created_at"]),
            updated_at=datetime.fromisoformat(json_resp["updated_at"]),
            dataset=json_resp["dataset"],
            model=json_resp["model"],
        )

    def __str__(self):
        output = "\n".join(
            [
                f"AutoNLP Evaluation (id # {self.evaluation_id})",
                "~" * 35,
                f" • {BOLD_TAG}Owner{RESET_TAG}:       {GREEN_TAG}{self.user}{RESET_TAG}",
                f" • {BOLD_TAG}Task{RESET_TAG}:        {YELLOW_TAG}{self.task.title().replace('_', ' ')}{RESET_TAG}",
                f" • {BOLD_TAG}Dataset{RESET_TAG}:       {GREEN_TAG}{self.dataset}{RESET_TAG}",
                f" • {BOLD_TAG}Model{RESET_TAG}:       {GREEN_TAG}{self.model}{RESET_TAG}",
                f" • {BOLD_TAG}Created at{RESET_TAG}:  {self.created_at.strftime('%Y-%m-%d %H:%M Z')}",
                f" • {BOLD_TAG}Last update{RESET_TAG}: {self.updated_at.strftime('%Y-%m-%d %H:%M Z')}",
            ]
        )
        return output


def get_eval_job_status(status_id: int) -> Tuple[str, str]:
    try:
        return EVAL_JOB_STATUS[status_id - 1]
    except IndexError:
        return "❓", "Unhandled status! Please update autonlp"


def format_eval_status(json_resp):
    status_emoji, status = get_eval_job_status(json_resp["status"])
    output = "\n".join(
        [
            f"AutoNLP Evaluation (id # {json_resp['id']})",
            "~" * 35,
            f" • {BOLD_TAG}Status{RESET_TAG}:       {status_emoji} {GREEN_TAG}{status}{RESET_TAG}",
        ]
    )
    return output


def format_datasets_task(task: str, dataset: str, config: str = None):
    task_template = get_compatible_task_template(task, dataset, config)
    if task_template:
        if task == "text-classification":
            num_labels = len(task_template.labels)
            if num_labels == 2:
                task = "binary_classification"
            # TODO(lewtun): add logic for multilabel classification when implemented in `datasets`
            elif num_labels > 2:
                task = "multi_class_classification"
            else:
                raise Exception("Invalid `num_labels`")
        elif task == "question-answering-extractive":
            task = "extractive_question_answering"
        else:
            raise ValueError(f"❌ Dataset `{dataset}` with configuration `{config}` is not suitable for `{task}`!")
    else:
        raise ValueError(
            f"❌ Dataset `{dataset}` with configuration `{config}` does not have a task template for `{task}`! Please select a different task and/or dataset."
        )
    return task


def get_compatible_task_template(task: str, dataset: str, config: str = None):
    module, module_hash = prepare_module(dataset)
    builder_cls = import_main_class(module)
    builder = builder_cls(hash=module_hash, name=config)
    templates = builder.info.task_templates
    if templates:
        compatible_templates = [template for template in templates if template.task == task]
        if not compatible_templates:
            raise ValueError(f"❌ Task `{task}` is not compatible with dataset `{dataset}`!")
        if len(compatible_templates) > 1:
            raise ValueError(
                f"❌ Expected 1 task template but found {len(compatible_templates)}! Please ensure that `datasets.DatasetInfo.task_templates` contains a unique set of task types."
            )
        return compatible_templates[0]
    else:
        return None


def get_dataset_splits(dataset: str, config: str = None):
    module, module_hash = prepare_module(dataset)
    builder_cls = import_main_class(module)
    builder = builder_cls(hash=module_hash, name=config)
    splits = builder.info.splits.keys()
    return list(splits)



import csv
import json
import os
from functools import partial
from typing import Dict, Iterable, TextIO


SUPPORTED_AUDIO_FILE_FORMAT = ("mp3", "wav")


def audio_file_name_iter(transcription_file_path: str, col_mapping: Dict[str, str]) -> Iterable[str]:
    filename = os.path.basename(transcription_file_path)
    file_ext = filename.split(".")[-1]
    path_column = [src for src, dst in col_mapping.items() if dst == "path"][0]

    if file_ext in ("csv", "tsv"):
        delimiter = "\t" if file_ext == "tsv" else ","
        sample_iterator = partial(csv.DictReader, delimiter=delimiter)
    else:
        sample_iterator = json_line_iterator

    with open(transcription_file_path, encoding="utf-8") as f:
        for sample in sample_iterator(f):
            path = sample[path_column]
            yield os.path.basename(path)


def json_line_iterator(f: TextIO) -> Iterable[dict]:
    for line in f:
        yield json.loads(line)



TASKS = {
    "binary_classification": 1,
    "multi_class_classification": 2,
    "entity_extraction": 4,
    "extractive_question_answering": 5,
    "summarization": 8,
    "single_column_regression": 10,
    "speech_recognition": 11,
}

DATASETS_TASKS = ["text-classification", "question-answering-extractive"]



import json
from dataclasses import dataclass

from prettytable import PrettyTable

from .utils import BOLD_TAG, RESET_TAG, http_get, http_post


@dataclass
class Metrics:
    _token: str
    username: str
    project_name: str
    project_id: int

    @classmethod
    def from_json_resp(cls, json_resp: dict, token: str, project_name: str, username: str):
        return cls(
            project_id=json_resp["id"],
            _token=token,
            project_name=project_name,
            username=username,
        )

    def print(self):
        payload = {"project_id": self.project_id, "username": self.username}
        resp = http_post(path="/models/metrics", token=self._token, suppress_logs=True, payload=payload)
        json_jobs = json.loads(resp.json())
        if len(json_jobs) > 0:
            columns = json_jobs.keys()
            print_logs = PrettyTable([col for col in columns])
            print_logs.title = f"{BOLD_TAG}🚀 AutoNLP Model Metrics for project: {self.project_name}{RESET_TAG}"
            indexes = json_jobs["model_id"].keys()
            for idx in indexes:
                line = [v[idx] if k == "model_id" else float(v[idx]) for k, v in json_jobs.items()]
                print_logs.add_row(line)

            print_logs = print_logs.get_string(sortby="eval_loss")
            print(print_logs)
            print("")
            resp = http_get(path=f"/zeus/cost/{self.project_id}", token=self._token, suppress_logs=True).json()
            print(f"Total cost incurred: USD {resp.get('cost_usd'):.2f}")
        else:
            print("No models have finished training yet!")



from typing import Any, Dict, Optional, Union

import requests

from . import config
from .tasks import TASKS


FORMAT_TAG = "\033[{code}m"
RESET_TAG = FORMAT_TAG.format(code=0)
BOLD_TAG = FORMAT_TAG.format(code=1)
RED_TAG = FORMAT_TAG.format(code=91)
GREEN_TAG = FORMAT_TAG.format(code=92)
YELLOW_TAG = FORMAT_TAG.format(code=93)
PURPLE_TAG = FORMAT_TAG.format(code=95)
CYAN_TAG = FORMAT_TAG.format(code=96)


class UnauthenticatedError(Exception):
    pass


class UnreachableAPIError(Exception):
    pass


def get_auth_headers(token: str, prefix: str = "autonlp"):
    # return {"Authorization": f"autonlp {token}"}
    return {"Authorization": f"{prefix} {token}"}


def http_get(
    path: str,
    token: str,
    domain: str = config.HF_AUTONLP_BACKEND_API,
    token_prefix: str = "autonlp",
    suppress_logs: bool = False,
    **kwargs,
) -> requests.Response:
    """HTTP GET request to the AutoNLP API, raises UnreachableAPIError if the API cannot be reached"""
    try:
        response = requests.get(
            url=domain + path, headers=get_auth_headers(token=token, prefix=token_prefix), **kwargs
        )
    except requests.exceptions.ConnectionError:
        raise UnreachableAPIError("❌ Failed to reach AutoNLP API, check your internet connection")
    response.raise_for_status()
    return response


def http_post(
    path: str,
    token: str,
    payload: Optional[Dict] = None,
    domain: str = config.HF_AUTONLP_BACKEND_API,
    suppress_logs: bool = False,
    **kwargs,
) -> requests.Response:
    """HTTP POST request to the AutoNLP API, raises UnreachableAPIError if the API cannot be reached"""
    try:
        response = requests.post(
            url=domain + path, json=payload, headers=get_auth_headers(token=token), allow_redirects=True, **kwargs
        )
    except requests.exceptions.ConnectionError:
        raise UnreachableAPIError("❌ Failed to reach AutoNLP API, check your internet connection")
    response.raise_for_status()
    return response


def get_task(task_id: int) -> str:
    for key, value in TASKS.items():
        if value == task_id:
            return key
    return "❌ Unsupported task! Please update autonlp"


def flatten_dict(dictionary: Dict[str, Union[str, Any]], max_depth: int):
    """Recursively flattens a dict"""
    flat_dict = {}

    def _flatten(dictionary: Dict[str, Union[str, Any]], max_depth: int, parent=None):
        for key, value in dictionary.items():
            flat_key = key if parent is None else ".".join([parent, key])
            if max_depth and isinstance(value, dict):
                _flatten(value, max_depth - 1, flat_key)
            else:
                flat_dict[flat_key] = value

    _flatten(dictionary, max_depth, None)
    return flat_dict



from ..utils import BOLD_TAG as BLD
from ..utils import CYAN_TAG as CYN
from ..utils import GREEN_TAG as GRN
from ..utils import RESET_TAG as RST


COL_MAPPING_ARG_HELP = f"""\
The files' column mapping. Must be like this:
'{GRN}col_name{RST}:{CYN}autonlp_col_name{RST},{GRN}col_name{RST}:{CYN}autonlp_col_name{RST}'
where '{CYN}autonlp_col_name{RST}' corresponds to an expected column in AutoNLP, and
'{GRN}col_name{RST}' is the corresponding column in your files.
"""

COL_MAPPING_HELP = f"""\
Expected columns for AutoNLP evaluation tasks:
--------------------------------------------------------

{BLD}col_name1{RST} and {BLD}col_name2{RST} refer to columns in your files.

{BLD}`binary_classification`{RST}:
    {BLD}col_name1{RST} -> {BLD}text{RST}    (The text to classify)
    {BLD}col_name2{RST} -> {BLD}target{RST}  (The label)
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}text{RST},{GRN}col_name2{RST}:{CYN}target{RST}'

{BLD}`multi_class_classification`{RST}:
    {BLD}col_name1{RST} -> {BLD}text{RST}    (The text to classify)
    {BLD}col_name2{RST} -> {BLD}target{RST}  (The label)
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}text,{GRN}col_name2{RST}:{CYN}target{RST}'

{BLD}`entity_extraction`{RST}:
    {BLD}col_name1{RST} -> {BLD}tokens{RST}  (The tokens to tag)
    {BLD}col_name2{RST} -> {BLD}tags{RST}    (The associated tags)
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}tokens{RST},{GRN}col_name2{RST}:{CYN}tags{RST}'

{BLD}`speech_recognition`{RST}:
    {BLD}col_name1{RST} -> {BLD}path{RST}  (The path to the audio file, only the file name matters)
    {BLD}col_name2{RST} -> {BLD}text{RST}  (The matching speech transcription)
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}path{RST},{GRN}col_name2{RST}:{CYN}text{RST}'

{BLD}`summarization`{RST}:
    {BLD}col_name1{RST} -> {BLD}text{RST}    (The text to summarize)
    {BLD}col_name2{RST} -> {BLD}target{RST}  (The summarization)
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}text{RST},{GRN}col_name2{RST}:{CYN}target{RST}'

{BLD}`extractive_question_answering`{RST}:
    {BLD}col_name1{RST} -> {BLD}context{RST}    (The context text)
    {BLD}col_name2{RST} -> {BLD}question{RST}  (The question text)
    {BLD}col_name3{RST} -> {BLD}answers.answer_start{RST}  (Character indices for the start of answers [a list of ints])
    {BLD}col_name2{RST} -> {BLD}answers.text{RST}  (Actual answer texts [a list of strings])
    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}context{RST},{GRN}col_name2{RST}:{CYN}question{RST},{GRN}col_name2{RST}:{CYN}answers.answer_start{RST},{GRN}col_name2{RST}:{CYN}answers.text{RST}'

"""



from argparse import ArgumentParser

from loguru import logger

from . import BaseAutoNLPCommand


def create_benchmark_command_factory(args):
    return CreateBenchmarkCommand(args.dataset, args.submission, args.eval_name)


class CreateBenchmarkCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        create_benchmark_parser = parser.add_parser("benchmark", description="✨ Creates an evaluation in AutoNLP.")
        create_benchmark_parser.add_argument(
            "--dataset",
            metavar="DATASET",
            type=str,
            default=None,
            required=True,
        )
        create_benchmark_parser.add_argument(
            "--submission",
            metavar="SUBMISSION",
            type=str,
            default=None,
            required=True,
        )
        create_benchmark_parser.add_argument(
            "--eval_name",
            metavar="EVAL_NAME",
            type=str,
            default=None,
            required=True,
        )

        create_benchmark_parser.set_defaults(func=create_benchmark_command_factory)

    def __init__(self, dataset, submission, eval_name):
        self._dataset = dataset
        self._submission = submission
        self._eval_name = eval_name

    def run(self):
        from ..autonlp import AutoNLP

        logger.info("Creating benchmark")
        client = AutoNLP()
        eval_project = client.create_benchmark(
            dataset=self._dataset,
            submission=self._submission,
            eval_name=self._eval_name,
        )
        print(eval_project)



import os
import sys

from loguru import logger


HF_AUTONLP_BACKEND_API = os.getenv("HF_AUTONLP_BACKEND_API", "https://api.autonlp.huggingface.co")

HF_API = os.getenv("HF_API", "https://huggingface.co/api")

logger.configure(handlers=[dict(sink=sys.stderr, format="> <level>{level:<7} {message}</level>")])



from argparse import ArgumentParser

from loguru import logger

from ..tasks import DATASETS_TASKS, TASKS
from . import BaseAutoNLPCommand
from .common import COL_MAPPING_HELP


def create_evaluation_command_factory(args):
    if args.task not in DATASETS_TASKS:
        if args.col_mapping is None:
            raise Exception("`col_mapping` is required if task is not a datasets task")
    return CreateEvaluationCommand(args.task, args.dataset, args.model, args.col_mapping, args.split, args.config)


class CreateEvaluationCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        create_evaluation_parser = parser.add_parser("evaluate", description="✨ Creates an evaluation in AutoNLP.")
        create_evaluation_parser.add_argument(
            "--task",
            metavar="TASK",
            type=str,
            default=None,
            required=True,
            help=f"The evaluation task type, one of: {list(TASKS.keys())}",
            choices=list(TASKS.keys()) + DATASETS_TASKS,
        )
        create_evaluation_parser.add_argument(
            "--dataset",
            metavar="DATASET",
            type=str,
            default=None,
            required=True,
            help="Dataset from the hub or from datasets libray. e.g. 'imdb', 'username/awesome_dataset', etc",
        )
        create_evaluation_parser.add_argument(
            "--model",
            metavar="MODEL",
            type=str,
            default=None,
            required=True,
            help="Model from the hub, e.g. 'bert-base-uncased', 'facebook/bart-large-mnli', etc. Model must be compatible with task and the dataset",
        )
        create_evaluation_parser.add_argument(
            "--col_mapping",
            type=str,
            default=None,
            required=False,
            help=COL_MAPPING_HELP,
        )
        create_evaluation_parser.add_argument(
            "--split",
            type=str,
            default="test",
            required=False,
            help="Which split of dataset to use for evaluation. If not provided, this will default to 'test'.",
        )
        create_evaluation_parser.add_argument(
            "--config",
            type=str,
            default=None,
            required=False,
            help="Which config of dataset to use for evaluation. If not provided, this will default to None.",
        )

        create_evaluation_parser.set_defaults(func=create_evaluation_command_factory)

    def __init__(self, task: str, dataset: str, model: str, col_mapping: str, split: str, config: str = None):
        self._task = task
        self._model = model
        self._dataset = dataset
        self._col_mapping = col_mapping
        self._split = split
        self._config = config

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Creating evaluation for task: {self._task}")
        client = AutoNLP()
        eval_project = client.create_evaluation(
            task=self._task,
            dataset=self._dataset,
            model=self._model,
            split=self._split,
            col_mapping=self._col_mapping,
            config=self._config,
        )
        print(eval_project)



import sys
from argparse import ArgumentParser

from loguru import logger

from ..project import TrainingCancelledError
from ..utils import CYAN_TAG as CYN
from ..utils import RED_TAG as RED
from ..utils import RESET_TAG as RST
from . import BaseAutoNLPCommand


def train_command_factory(args):
    return TrainCommand(args.project)


class TrainCommand(BaseAutoNLPCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        train_parser = parser.add_parser("train", description="🚀 Start the training for your project!")
        train_parser.add_argument("--project", type=str, default=None, required=True, help="The project name")
        train_parser.set_defaults(func=train_command_factory)

    def __init__(self, name: str):
        self._name = name

    def run(self):
        from ..autonlp import AutoNLP

        logger.info(f"Starting Training For Project: {self._name}")

        client = AutoNLP()
        try:
            project = client.get_project(name=self._name)
        except ValueError:
            logger.error(f"Project {self._name} not found! You can create it using the create_project command.")
            sys.exit(1)
        try:
            project.train()
            print(
                f"🚀 Awesome!! Monitor you training progress here: {RED}autonlp project_info --name {project.name}{RST}"
            )
        except TrainingCancelledError:
            print(
                f"\n☹ Training cancelled! Tell us why on GitHub: {CYN}https://github.com/huggingface/autonlp/issues/new{RST}"
            )
